(* File generated from abstract1.idl *)

(* This file is part of the APRON Library, released under LGPL license.
   Please read the COPYING file packaged in the distribution  *)

type t = {
  mutable abstract0: Abstract0.t;
  mutable env: Environment.t;
}

type box1 = { mutable interval_array : Interval.t array; mutable box1_env : Environment.t }
(* ********************************************************************** *)
(** {2 General management} *)
(* ********************************************************************** *)
(* ============================================================ *)
(** {3 Memory} *)
(* ============================================================ *)

let copy man x = { abstract0 = Abstract0.copy man x.abstract0; env = x.env }
let size man x = Abstract0.size man x.abstract0

(* ============================================================ *)
(** {3 Control of internal representation} *)
(* ============================================================ *)

let minimize man x = Abstract0.minimize man x.abstract0
let canonicalize man x = Abstract0.canonicalize man x.abstract0
let approximate man x n = Abstract0.approximate man x.abstract0
let is_minimal man x = Abstract0.is_minimal man x.abstract0
let is_canonical man x = Abstract0.is_canonical man x.abstract0

(* ============================================================ *)
(** {3 Printing} *)
(* ============================================================ *)
external fdump : Manager.t -> t -> unit
	= "camlidl_abstract1_ap_abstract1_fdump"

(* ============================================================ *)
(** {3 Serialization} *)
(* ============================================================ *)
(* ********************************************************************** *)
(** {2 Constructor, accessors, tests and property extraction} *)
(* ********************************************************************** *)
(* ============================================================ *)
(** {3 Basic constructors} *)

(* ============================================================ *)
external bottom : Manager.t -> Environment.t -> t
	= "camlidl_abstract1_ap_abstract1_bottom"

external top : Manager.t -> Environment.t -> t
	= "camlidl_abstract1_ap_abstract1_top"

external of_box : Manager.t -> Environment.t -> Var.t array -> Interval.t array -> t
	= "camlidl_abstract1_ap_abstract1_of_box"

external of_lincons_array : Manager.t -> Environment.t -> Lincons1.earray -> t
	= "camlidl_abstract1_ap_abstract1_of_lincons_array"

(* ============================================================ *)
(** {3 Accessors} *)

(* ============================================================ *)

let manager x = Abstract0.manager x.abstract0
let env x = x.env
let abstract0 x = x.abstract0

(* ============================================================ *)
(** {3 Tests} *)

(* ============================================================ *)

let is_bottom man x = Abstract0.is_bottom man x.abstract0
let is_top man x = Abstract0.is_top man x.abstract0

external is_leq : Manager.t -> t -> t -> Manager.tbool
	= "camlidl_abstract1_ap_abstract1_is_leq"

external is_eq : Manager.t -> t -> t -> Manager.tbool
	= "camlidl_abstract1_ap_abstract1_is_eq"

external sat_lincons : Manager.t -> t -> Lincons1.t -> Manager.tbool
	= "camlidl_abstract1_ap_abstract1_sat_lincons"

external sat_interval : Manager.t -> t -> Var.t -> Interval.t -> Manager.tbool
	= "camlidl_abstract1_ap_abstract1_sat_interval"

external is_variable_unconstrained : Manager.t -> t -> Var.t -> Manager.tbool
	= "camlidl_abstract1_ap_abstract1_is_variable_unconstrained"

(* ============================================================ *)
(** {3 Extraction of properties} *)

(* ============================================================ *)
external bound_variable : Manager.t -> t -> Var.t -> Interval.t
	= "camlidl_abstract1_ap_abstract1_bound_variable"

external bound_linexpr : Manager.t -> t -> Linexpr1.t -> Interval.t
	= "camlidl_abstract1_ap_abstract1_bound_linexpr"


let to_lincons_array man x = {
Lincons1.lincons0_array = Abstract0.to_lincons_array man x.abstract0;
Lincons1.array_env = x.env
}
let to_generator_array man x = {
Generator1.generator0_array = Abstract0.to_generator_array man x.abstract0;
Generator1.array_env = x.env
}
let to_box man x = {
interval_array = Abstract0.to_box man x.abstract0;
box1_env = x.env
}

(* ********************************************************************** *)
(** {2 Operations} *)
(* ********************************************************************** *)
(* ============================================================ *)
(** {3 Meet and Join} *)

(* ============================================================ *)
external meet : Manager.t -> t -> t -> t
	= "camlidl_abstract1_ap_abstract1_meet"

external meet_array : Manager.t -> t array -> t
	= "camlidl_abstract1_ap_abstract1_meet_array"

external meet_lincons_array : Manager.t -> t -> Lincons1.earray -> t
	= "camlidl_abstract1_ap_abstract1_meet_lincons_array"

external join : Manager.t -> t -> t -> t
	= "camlidl_abstract1_ap_abstract1_join"

external join_array : Manager.t -> t array -> t
	= "camlidl_abstract1_ap_abstract1_join_array"

external add_ray_array : Manager.t -> t -> Generator1.earray -> t
	= "camlidl_abstract1_ap_abstract1_add_ray_array"

external meet_with : Manager.t -> t -> t -> unit
	= "camlidl_abstract1_ap_abstract1_meet_with"

external meet_lincons_array_with : Manager.t -> t -> Lincons1.earray -> unit
	= "camlidl_abstract1_ap_abstract1_meet_lincons_array_with"

external join_with : Manager.t -> t -> t -> unit
	= "camlidl_abstract1_ap_abstract1_join_with"

external add_ray_array_with : Manager.t -> t -> Generator1.earray -> unit
	= "camlidl_abstract1_ap_abstract1_add_ray_array_with"

(* ============================================================ *)
(** {3 Assignement and Substitutions} *)

(* ============================================================ *)
external assign_linexpr : Manager.t -> t -> Var.t -> Linexpr1.t -> t option -> t
	= "camlidl_abstract1_ap_abstract1_assign_linexpr"

external substitute_linexpr : Manager.t -> t -> Var.t -> Linexpr1.t -> t option -> t
	= "camlidl_abstract1_ap_abstract1_substitute_linexpr"

external assign_linexpr_array : Manager.t -> t -> Var.t array -> Linexpr1.t array -> t option -> t
	= "camlidl_abstract1_ap_abstract1_assign_linexpr_array"

external substitute_linexpr_array : Manager.t -> t -> Var.t array -> Linexpr1.t array -> t option -> t
	= "camlidl_abstract1_ap_abstract1_substitute_linexpr_array"

external assign_linexpr_with : Manager.t -> t -> Var.t -> Linexpr1.t -> t option -> unit
	= "camlidl_abstract1_ap_abstract1_assign_linexpr_with"

external substitute_linexpr_with : Manager.t -> t -> Var.t -> Linexpr1.t -> t option -> unit
	= "camlidl_abstract1_ap_abstract1_substitute_linexpr_with"

external assign_linexpr_array_with : Manager.t -> t -> Var.t array -> Linexpr1.t array -> t option -> unit
	= "camlidl_abstract1_ap_abstract1_assign_linexpr_array_with"

external substitute_linexpr_array_with : Manager.t -> t -> Var.t array -> Linexpr1.t array -> t option -> unit
	= "camlidl_abstract1_ap_abstract1_substitute_linexpr_array_with"

(* ============================================================ *)
(** {3 Projections} *)

(* ============================================================ *)
external forget_array : Manager.t -> t -> Var.t array -> bool -> t
	= "camlidl_abstract1_ap_abstract1_forget_array"

external forget_array_with : Manager.t -> t -> Var.t array -> bool -> unit
	= "camlidl_abstract1_ap_abstract1_forget_array_with"

(* ============================================================ *)
(** {3 Change and permutation of dimensions} *)

(* ============================================================ *)
external change_environment : Manager.t -> t -> Environment.t -> bool -> t
	= "camlidl_abstract1_ap_abstract1_change_environment"

external minimize_environment : Manager.t -> t -> t
	= "camlidl_abstract1_ap_abstract1_minimize_environment"

external rename_array : Manager.t -> t -> Var.t array -> Var.t array -> t
	= "camlidl_abstract1_ap_abstract1_rename_array"

external change_environment_with : Manager.t -> t -> Environment.t -> bool -> unit
	= "camlidl_abstract1_ap_abstract1_change_environment_with"

external minimize_environment_with : Manager.t -> t -> unit
	= "camlidl_abstract1_ap_abstract1_minimize_environment_with"

external rename_array_with : Manager.t -> t -> Var.t array -> Var.t array -> unit
	= "camlidl_abstract1_ap_abstract1_rename_array_with"

(* ============================================================ *)
(** {3 Expansion and folding of dimensions} *)

(* ============================================================ *)
external expand : Manager.t -> t -> Var.t -> Var.t array -> t
	= "camlidl_abstract1_ap_abstract1_expand"

external fold : Manager.t -> t -> Var.t array -> t
	= "camlidl_abstract1_ap_abstract1_fold"

external expand_with : Manager.t -> t -> Var.t -> Var.t array -> unit
	= "camlidl_abstract1_ap_abstract1_expand_with"

external fold_with : Manager.t -> t -> Var.t array -> unit
	= "camlidl_abstract1_ap_abstract1_fold_with"

(* ============================================================ *)
(** {3 Widening} *)

(* ============================================================ *)
external widening : Manager.t -> t -> t -> t
	= "camlidl_abstract1_ap_abstract1_widening"

external widening_threshold : Manager.t -> t -> t -> Lincons1.earray -> t
	= "camlidl_abstract1_ap_abstract1_widening_threshold"

(* ============================================================ *)
(** {3 Closure operation} *)

(* ============================================================ *)
external closure : Manager.t -> t -> t
	= "camlidl_abstract1_ap_abstract1_closure"

external closure_with : Manager.t -> t -> unit
	= "camlidl_abstract1_ap_abstract1_closure_with"


let print fmt a =
let man = manager a in
if is_bottom man a = Manager.True then
Format.pp_print_string fmt "bottom"
else if is_top man a = Manager.True then
Format.pp_print_string fmt "top"
else begin
let tab = to_lincons_array man a in
Lincons1.array_print fmt tab;
end

