(* File generated from manager.idl *)

(* This file is part of the APRON Library, released under LGPL license.
   Please read the COPYING file packaged in the distribution  *)

type tbool =
  | False
  | True
  | Top
and funid =
  | Funid_unknown
  | Funid_copy
  | Funid_free
  | Funid_asize
  | Funid_minimize
  | Funid_canonicalize
  | Funid_approximate
  | Funid_is_minimal
  | Funid_is_canonical
  | Funid_fprint
  | Funid_fprintdiff
  | Funid_fdump
  | Funid_serialize_raw
  | Funid_deserialize_raw
  | Funid_bottom
  | Funid_top
  | Funid_of_box
  | Funid_of_lincons_array
  | Funid_dimension
  | Funid_is_bottom
  | Funid_is_top
  | Funid_is_leq
  | Funid_is_eq
  | Funid_is_dimension_unconstrained
  | Funid_sat_interval
  | Funid_sat_lincons
  | Funid_bound_dimension
  | Funid_bound_linexpr
  | Funid_to_box
  | Funid_to_lincons_array
  | Funid_to_generator_array
  | Funid_meet
  | Funid_meet_array
  | Funid_meet_lincons_array
  | Funid_join
  | Funid_join_array
  | Funid_add_ray_array
  | Funid_assign_linexpr
  | Funid_assign_linexpr_array
  | Funid_substitute_linexpr
  | Funid_substitute_linexpr_array
  | Funid_add_dimensions
  | Funid_remove_dimensions
  | Funid_permute_dimensions
  | Funid_forget_array
  | Funid_expand
  | Funid_fold
  | Funid_widening
  | Funid_closure
  | Funid_size
  | Funid_change_environment
  | Funid_rename_array
  | Funid_size2
and exc =
  | Exc_none
  | Exc_timeout
  | Exc_out_of_space
  | Exc_overflow
  | Exc_invalid_argument
  | Exc_not_implemented
  | Exc_size
and funopt = {
  algorithm: int;
  approx_before: int;
  approx_after: int;
  timeout: int;
  max_object_size: int;
  flag_exact_wanted: bool;
  flag_best_wanted: bool;
}
and exclog = {
  exn: exc;
  funid: funid;
  msg: string;
}
and t

(** APRON Managers *)
(** Get the name of the effective library which allocated the manager *)
external get_library : t -> string
	= "camlidl_manager_ap_manager_get_library"

(** Get the version of the effective library which allocated the manager *)
external get_version : t -> string
	= "camlidl_manager_ap_manager_get_version"

(** Return the default options for any function ([0] or [false] for al fields) *)
external funopt_make : unit -> funopt
	= "camlidl_manager_ap_funopt_make"

(** Get the options sets for the function. The result is a copy of the internal structure and may be freely modified *)
external get_funopt : t -> funid -> funopt
	= "camlidl_manager_ap_manager_get_funopt"

(** Set the options for the function *)
external set_funopt : t -> funid -> funopt -> unit
	= "camlidl_manager_ap_manager_set_funopt"

(** Get the corresponding result flags *)
external get_flag_exact : t -> tbool
	= "camlidl_manager_ap_manager_get_flag_exact"

external get_flag_best : t -> tbool
	= "camlidl_manager_ap_manager_get_flag_best"

(** Exception raised by functions of the interface *)
exception Error of exclog
val string_of_tbool: tbool -> string
val string_of_funid: funid -> string
val string_of_exc: exc -> string
val print_tbool: Format.formatter -> tbool -> unit
(** Set / get the global manager used for deserialization *)
external set_deserialize : t -> unit
	= "camlidl_manager_ap_manager_set_deserialize"

external get_deserialize : unit -> t
	= "camlidl_manager_ap_manager_get_deserialize"

