(* File generated from scalar.idl *)

(* This file is part of the APRON Library, released under LGPL license.
   Please read the COPYING file packaged in the distribution  *)

type t = Float of float | Mpqf of Mpqf.t



(** APRON Scalar numbers. See {!Mpqf} for operations on GMP multiprecision rational numbers. *)


let of_mpq x = Mpqf (Mpqf.of_mpq x)
let of_mpqf x = Mpqf x
let of_int x = Mpqf(Mpqf.of_int x)
let of_frac x y = Mpqf(Mpqf.of_frac x y)
let of_float x = Float(x)
let of_infty s = 
if s>0 then Float(Pervasives.infinity)
else if s<0 then Float(Pervasives.neg_infinity)
else Float(0.0)
let is_infty scalar =
match scalar with
| Mpqf x ->
let z = Mpqf.get_den x in
if Mpzf.sgn z <> 0 then 0
else begin
let z = Mpqf.get_num x in
let sgn = Mpzf.sgn z in
if sgn > 0 then 1 else if sgn < 0 then -1 else 0
end
| Float x ->
if x = Pervasives.infinity then 1
else if x = Pervasives.neg_infinity then -1
else 0
let sgn scalar =
match scalar with
| Mpqf x -> Mpqf.sgn x
| Float x -> if x > 0.0+.0.0 then 1 else if x < -.0.0 then -1 else 0
let cmp c1 c2 =
let s1 = is_infty c1 in
let s2 = is_infty c2 in
if s1>s2 then 1
else if s1<s2 then -1
else if s1 != 0 then 0
else begin
match (c1,c2) with
| (Mpqf x1, Mpqf x2) -> Mpqf.cmp x1 x2
| (Float x1, Float x2) ->
if x1>x2 then 1 else if x1<x2 then -1 else 0
| (Float f, Mpqf x2) ->
let x1 = Mpqf.of_float f in
Mpqf.cmp x1 x2
| (Mpqf x1, Float f) ->
let x2 = Mpqf.of_float f in
Mpqf.cmp x1 x2
end
let equal c1 c2 =
let s1 = is_infty c1 in
let s2 = is_infty c2 in
if s1!=s2 then false
else if s1!=0 then true
else begin
match (c1,c2) with
| (Mpqf x1, Mpqf x2) -> Mpqf.equal x1 x2
| (Float x1, Float x2) -> x1 = x2
| (Float f, Mpqf x2) ->
let x1 = Mpqf.of_float f in
Mpqf.equal x1 x2
| (Mpqf x1, Float f) ->
let x2 = Mpqf.of_float f in
Mpqf.equal x1 x2
end
let cmp_int scalar n =
match scalar with
| Mpqf x -> Mpqf.cmp_int x n
| Float x -> Pervasives.compare x (float_of_int n)
let equal_int scalar n =
match scalar with
| Mpqf x -> (Mpqf.cmp_int x n)=0
| Float x -> x==(float_of_int n)
let neg scalar =
match scalar with
| Mpqf x -> Mpqf(Mpqf.neg x)
| Float x -> Float(-. x)
let to_string scalar =
match scalar with
| Mpqf x -> Mpqf.to_string x
| Float x -> string_of_float x
let print fmt scalar =
Format.pp_print_string fmt (to_string scalar)

